package com.gree.mall.manager.logic.settle.repair;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.settle.repair.DailyBankAccountVO;
import com.gree.mall.manager.commonmapper.CommonMapper;
import com.gree.mall.manager.enums.ExamineStatusEnum;
import com.gree.mall.manager.enums.UserTypeEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.AdminWebsit;
import com.gree.mall.manager.plus.entity.SettleDailyBankAccount;
import com.gree.mall.manager.plus.entity.User;
import com.gree.mall.manager.plus.entity.WebsitUser;
import com.gree.mall.manager.plus.service.AdminWebsitService;
import com.gree.mall.manager.plus.service.SettleDailyBankAccountService;
import com.gree.mall.manager.plus.service.UserService;
import com.gree.mall.manager.plus.service.WebsitUserService;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class RepairSettleAccountLogic {
    private final CommonLogic commonLogic;
    private final CommonMapper commonMapper;
    private final SettleDailyBankAccountService settleDailyBankAccountService;
    private final UserService userService;
    private final WebsitUserService websitUserService;
    private final AdminWebsitService adminWebsitService;

    public IPage<DailyBankAccountVO> page(ZfireParamBean zfireParamBean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        FieldUtils.supplyParam(zfireParamBean, DailyBankAccountVO.class, adminUser);

        return commonMapper.repairSettleBankAccountList(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
    }

    public SettleDailyBankAccount detail(String id) {
        return settleDailyBankAccountService.getById(id);
    }


    public void add(SettleDailyBankAccount dailyBankAccount) {
        AdminUserCom adminUser = commonLogic.getAdminUser();

        if (adminUser.getType() == 2) {
            throw new RemoteServiceException("平台账号禁止操作");
        }

        String idcard = dailyBankAccount.getIdcard();
        List<SettleDailyBankAccount> count = settleDailyBankAccountService.lambdaQuery().eq(SettleDailyBankAccount::getIdcard, idcard).list();
        if (count.size() > 0) {
            if (count.get(0).getIdcard().equals(dailyBankAccount.getIdcard())) {
                throw new RemoteServiceException("身份证已存在:" + idcard);
            }

            if (count.get(0).getBankAccount().equals(dailyBankAccount.getBankAccount())) {
                throw new RemoteServiceException("银行卡已存在:" + idcard);
            }
        }

        List<User> userList = userService.lambdaQuery()
                .eq(User::getCompanyWechatId, adminUser.getAdminCompanyWechat().getCompanyWechatId())
                .eq(User::getIdCard, dailyBankAccount.getIdcard())
                .eq(User::getType, UserTypeEnum.WORKER.getKey())
                .list();
        if (CollectionUtil.isEmpty(userList)) {
            throw new RemoteServiceException("身份证在系统不存在:" + idcard);
        }
        if (!dailyBankAccount.getDepositBank().contains("招商银行") && StringUtils.isBlank(dailyBankAccount.getBankAddr())) {
            throw new RemoteServiceException("卡号【" + dailyBankAccount.getBankAccount() + "】非招商银行，请补充开户行地址");
        }
        dailyBankAccount.setBankAccount(dailyBankAccount.getBankAccount().replaceAll(" ", ""));

        List<SettleDailyBankAccount> accountList = new ArrayList<>();
        // 检查师傅已入驻网点并创建相应网点记录
        final List<WebsitUser> websitUserList = websitUserService.lambdaQuery()
                .eq(WebsitUser::getCompanyWechatId, adminUser.getAdminCompanyWechat().getCompanyWechatId())
                .eq(WebsitUser::getExamineStatus, ExamineStatusEnum.OK.getKey())
                .in(WebsitUser::getUserId, userList.stream().map(User::getUserId).distinct().collect(Collectors.toList()))
                .list();

        final List<String> websits = websitUserList.stream().map(WebsitUser::getWebsitId).distinct().collect(Collectors.toList());

        final List<AdminWebsit> adminWebsits = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getCompanyWechatId, adminUser.getAdminCompanyWechat().getCompanyWechatId())
                .in(AdminWebsit::getWebsitId, websits)
                .list();

        final Map<String, String> websitMap = adminWebsits.stream()
                .collect(Collectors.toMap(AdminWebsit::getWebsitId, AdminWebsit::getName));

        for (String websit : websits) {
            final String websitName = websitMap.get(websit);
            SettleDailyBankAccount account = new SettleDailyBankAccount();
            BeanUtils.copyProperties(dailyBankAccount, account);
            account.setCompanyWechatId(adminUser.getAdminCompanyWechat().getCompanyWechatId())
                    .setCompanyName(adminUser.getAdminCompanyWechat().getCompanyName())
                    .setWebsitId(websit)
                    .setWebsitName(websitName);
            accountList.add(account);
        }
    }

    public void update(SettleDailyBankAccount dailyBankAccount) {
    }

    public void delete(Integer id) {
    }
}
